using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;
using Microsoft.Xna.Framework.Net;
using Microsoft.Xna.Framework.Storage;
using System.IO;
using System.Reflection;

namespace WindowsGame1
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        RenderTarget2D renderTarget;
        Texture2D renderTex;

        SpriteFont fntTestFont;
        Effect effFont;
        Texture2D texWilyCastle;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            graphics.PreferredBackBufferWidth = 256 * 3;
            graphics.PreferredBackBufferHeight = 240 * 3;
            Content.RootDirectory = "Content";
        }

        private GameLevel m_currentLevel;

        internal GameLevel CurrentLevel
        {
            get { return m_currentLevel; }
            set
            {
                if (value != m_currentLevel)
                {
                    if (m_currentLevel != null)
                    {
                        m_currentLevel.UnloadContent(this.Content);
                    }
                    m_currentLevel = value;
                    if (m_currentLevel != null)
                    {
                        m_currentLevel.LoadContent(this.Content);
                    }
                }
            }
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            
            base.Initialize();
            StartLevel();
        }

        private void StartLevel()
        {
            using (Stream s = Assembly.GetExecutingAssembly().GetManifestResourceStream("WindowsGame1.TextFile1.txt"))
            using (StreamReader dr = new StreamReader(s))
            {
                this.CurrentLevel = GameLevel.CreateTestLevel(dr);
            }
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
            renderTarget = new RenderTarget2D(GraphicsDevice, 256, 240, 1, GraphicsDevice.DisplayMode.Format);

            texWilyCastle = Content.Load<Texture2D>("wilycastle");

            fntTestFont = Content.Load<SpriteFont>("TestFont");
            fntTestFont.Spacing = 1;

            effFont = Content.Load<Effect>("OneBitAlphaFont");
            
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            KeyboardState ks = Keyboard.GetState();
            if (ks.IsKeyDown(Keys.R))
            {
                StartLevel();
            }

            // TODO: Add your update logic here
            if (this.CurrentLevel != null)
            {
                this.CurrentLevel.Tick(gameTime);
            }

            base.Update(gameTime);
        }

        DateTime fcDumpDt = DateTime.Now;
        int frameCount = 0;
        System.Text.StringBuilder sbFpsCounter = new System.Text.StringBuilder();

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            DateTime thisFrameDt = DateTime.Now;
            if (thisFrameDt > fcDumpDt)
            {
                sbFpsCounter.Length = 0;
                sbFpsCounter.Append("FPS: ");
                sbFpsCounter.Append(frameCount);
                fcDumpDt = thisFrameDt + TimeSpan.FromSeconds(1.0);
                frameCount = 0;
            }
            frameCount++;

            GraphicsDevice.SetRenderTarget(0, renderTarget);
            GraphicsDevice.Clear(Color.CornflowerBlue);
            spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            if (this.CurrentLevel != null)
            {
                this.CurrentLevel.Draw(spriteBatch);
            }
            //spriteBatch.End();


            //// There's gotta be a better way of doing this

            //spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            //effFont.Parameters["PixColor"].SetValue(Color.Black.ToVector4());
            //effFont.Begin();
            //effFont.CurrentTechnique.Passes[0].Begin();
            ////spriteBatch.Draw(texWilyCastle, new Vector2(0.0f, 0.0f), Color.White);
            spriteBatch.DrawString(fntTestFont, sbFpsCounter, new Vector2(1.0f, 1.0f), Color.Black);
            //effFont.CurrentTechnique.Passes[0].End();
            //effFont.End();
            //spriteBatch.End();

            //spriteBatch.Begin(SpriteBlendMode.AlphaBlend, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            //effFont.Parameters["PixColor"].SetValue(Color.White.ToVector4());
            //effFont.Begin();
            //effFont.CurrentTechnique.Passes[0].Begin();
            spriteBatch.DrawString(fntTestFont, sbFpsCounter, new Vector2(0.0f, 0.0f), Color.White);
            //effFont.CurrentTechnique.Passes[0].End();
            //effFont.End();
            spriteBatch.End();

            

            GraphicsDevice.SetRenderTarget(0, null);
            renderTex = renderTarget.GetTexture();
            GraphicsDevice.Clear(Color.DarkBlue);
            spriteBatch.Begin(SpriteBlendMode.None, SpriteSortMode.Immediate, SaveStateMode.SaveState);
            GraphicsDevice.SamplerStates[0].MagFilter = TextureFilter.None;
            GraphicsDevice.SamplerStates[0].MinFilter = TextureFilter.None;
            GraphicsDevice.SamplerStates[0].MipFilter = TextureFilter.None;
            spriteBatch.Draw(renderTex, new Rectangle(0, 0, this.Window.ClientBounds.Width, this.Window.ClientBounds.Height), Color.White);
            spriteBatch.End();

            // TODO: Add your drawing code here

            base.Draw(gameTime);
        }
    }
}
