﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
using WindowsGame1.GameObjects;
using WindowsGame1.GameObjects.Powerups;
using WindowsGame1.GameObjects.Enemies;
using System.IO;

namespace WindowsGame1
{
    class GameLevel
    {
        public static GameLevel CreateTestLevel(TextReader r)
        {
            int maxx = 0;
            int maxy = 0;
            GameLevel res = new GameLevel();
            string tline;
            while ((tline = r.ReadLine()) != null)
            {
                for (int x = 0; x < tline.Length; x++)
                {
                    switch (tline[x])
                    {
                        case '#':
                            FloorTile ft = new FloorTile();
                            ft.TextureName = "floortile3";
                            ft.Location = new Point(x, maxy);
                            res.GameObjects.Add(ft);
                            break;
                        case '*':
                            SpikeTile st = new SpikeTile();
                            st.TextureName = "SpikeTile";
                            st.Location = new Point(x, maxy);
                            res.GameObjects.Add(st);
                            break;
                        case 'M':
                        case 'm':
                            PlayerObject p = new PlayerObject();
                            p.Location = new Point(x * 16, (maxy + 1) * 16);
                            //p.BoundingBox = new Rectangle(40, 40, 16, 22);
                            res.GameObjects.Add(p);
                            res.PlayerObject = p;
                            break;
                        case 'E':
                            EnergyTank et = new EnergyTank();
                            et.Location = new Point(x * 16 + (et.BoundingBox.Width / 2), (maxy + 1) * 16 - et.BoundingBox.Height);
                            res.GameObjects.Add(et);
                            break;
                        case 'H':
                            LargeHealthPellet lhp = new LargeHealthPellet();
                            lhp.Location = new Point(x * 16 + (lhp.BoundingBox.Width / 2), (maxy + 1) * 16 - lhp.BoundingBox.Height);
                            res.GameObjects.Add(lhp);
                            break;
                        case 'h':
                            SmallHealthPellet shp = new SmallHealthPellet();
                            shp.Location = new Point(x * 16 + (shp.BoundingBox.Width / 2), (maxy + 1) * 16 - shp.BoundingBox.Height);
                            res.GameObjects.Add(shp);
                            break;
                        case 'C':
                            CanEnemySpawner cn = new CanEnemySpawner();
                            cn.Location = new Point(x * 16, (maxy + 1) * 16);
                            res.GameObjects.Add(cn);
                            break;
                    }
                    maxx = Math.Max(maxx, x + 1);
                }
                maxy++;
            }

            res.CameraBounds = new Rectangle(0, 0, maxx * 16, maxy * 16);
            res.LevelStartDateTime = DateTime.Now;
            return res;

        }

        //public static GameLevel CreateTestLevel()
        //{
        //    const int maxx = 30;
        //    const int maxy = 30;
        //    GameLevel res = new GameLevel();
        //    for (int x = 0; x < maxx; x++)
        //    {
        //        for (int y = 0; y < maxy; y++)
        //        {
        //            BackgroundTile bgt = null;
        //            if ((x == 0 || x == maxx - 1 || y == 0 || y == maxy - 1) ||
        //                (y == maxy-5 && (x < 6 || x > maxx - 7)))
        //            {
        //                bgt = new FloorTile();
        //                bgt.TextureName = "floortile3";
        //            }
        //            else
        //            {
        //                //bgt = new BackgroundTile();
        //                //bgt.TextureName = "floortile2";
        //            }
        //            if (bgt != null)
        //            {
        //                bgt.Location = new Point(x, y);
        //                res.GameObjects.Add(bgt);
        //            }
        //        }
        //    }
        //    CanEnemy en = new CanEnemy();
        //    en.BoundingBox = new Rectangle((maxx - 4) * 16 + 5, (maxy - 7) * 16 + 5, en.BoundingBox.Width, en.BoundingBox.Height);
        //    res.GameObjects.Add(en);

        //    LargeHealthPellet lhp = new LargeHealthPellet();
        //    lhp.Location = new Point(70, 70);
        //    res.GameObjects.Add(lhp);

        //    SmallHealthPellet shp = new SmallHealthPellet();
        //    shp.Location = new Point(90, 70);
        //    res.GameObjects.Add(shp);

        //    EnergyTank et = new EnergyTank();
        //    et.Location = new Point(150, 66);
        //    res.GameObjects.Add(et);

        //    PlayerObject po = new PlayerObject();
        //    po.TextureName = "playerobject";
        //    po.BoundingBox = new Rectangle(40, 40, 16, 22);
        //    res.GameObjects.Add(po);
        //    res.PlayerObject = po;
        //    res.CameraBounds = new Rectangle(0, 0, maxx * 16, maxy * 16);

        //    res.LevelStartDateTime = DateTime.Now;
        //    return res;
        //}

        public GameLevel()
        {
            LevelGameObjectCollection go = new LevelGameObjectCollection(this);
            go.AttachIndex(this.GameObjectsByTickPriority);
            this.GameObjects = go;

            this.CameraLocation = new Point(0, 0);
            this.ScreenSize = new Point(256, 240);
            this.Gravity = new Vector2(0.0f, 0.6f);
        }

        public Vector2 Gravity { get; set; }


        public GameObjectCollection GameObjects { get; private set; }
        private TickPriorityIndex GameObjectsByTickPriority = new TickPriorityIndex();
        public Rectangle CameraBounds { get; set; }
        public Point CameraLocation { get; set; }
        public Point ScreenSize { get; set; }

        protected Point CurrentCameraTopLeft { get; private set; }
        public Rectangle CurrentCameraArea { get; private set; }

        internal ContentManager LevelContentManager;
        internal Dictionary<string, Texture2D> Textures = new Dictionary<string, Texture2D>();
        internal Dictionary<string, SoundEffect> SoundEffects = new Dictionary<string, SoundEffect>();

        public virtual void LoadContent(ContentManager content)
        {
            LevelContentManager = new ContentManager(content.ServiceProvider, content.RootDirectory);

            this.Textures.Add("floortile2", LevelContentManager.Load<Texture2D>("floortile2"));
            this.Textures.Add("floortile3", LevelContentManager.Load<Texture2D>("floortile3"));
            this.Textures.Add("SpikeTile", LevelContentManager.Load<Texture2D>("SpikeTile"));
            //this.Textures.Add("playerobject", LevelContentManager.Load<Texture2D>("playerobject"));
            //this.Textures.Add("playerobject2", LevelContentManager.Load<Texture2D>("playerobject2"));

            this.Textures.Add("Player.Meter.Background", LevelContentManager.Load<Texture2D>("Player\\Meter\\Player.Meter.Background"));
            this.Textures.Add("Player.Meter.HealthPip", LevelContentManager.Load<Texture2D>("Player\\Meter\\Player.Meter.HealthPip"));
            this.Textures.Add("Player.Sweat.1", LevelContentManager.Load<Texture2D>("Player\\Player.Sweat.1"));
            this.Textures.Add("Player.Sweat.2", LevelContentManager.Load<Texture2D>("Player\\Player.Sweat.2"));
            this.Textures.Add("Player.TookDamage", LevelContentManager.Load<Texture2D>("Player\\Player.TookDamage"));

            this.Textures.Add("Player.Stunned.1", LevelContentManager.Load<Texture2D>("Player\\Player.Stunned.1"));
            this.Textures.Add("Player.InAir.1", LevelContentManager.Load<Texture2D>("Player\\Player.InAir.1"));
            this.Textures.Add("Player.Running.0", LevelContentManager.Load<Texture2D>("Player\\Player.Running.0"));
            this.Textures.Add("Player.Running.1", LevelContentManager.Load<Texture2D>("Player\\Player.Running.1"));
            this.Textures.Add("Player.Running.2", LevelContentManager.Load<Texture2D>("Player\\Player.Running.2"));
            this.Textures.Add("Player.Running.3", LevelContentManager.Load<Texture2D>("Player\\Player.Running.3"));
            this.Textures.Add("Player.Standing.1", LevelContentManager.Load<Texture2D>("Player\\Player.Standing.1"));
            this.Textures.Add("Player.Standing.2", LevelContentManager.Load<Texture2D>("Player\\Player.Standing.2"));

            this.Textures.Add("Player.InAirFire.1", LevelContentManager.Load<Texture2D>("Player\\Player.InAirFire.1"));
            this.Textures.Add("Player.RunningFire.1", LevelContentManager.Load<Texture2D>("Player\\Player.RunningFire.1"));
            this.Textures.Add("Player.RunningFire.2", LevelContentManager.Load<Texture2D>("Player\\Player.RunningFire.2"));
            this.Textures.Add("Player.RunningFire.3", LevelContentManager.Load<Texture2D>("Player\\Player.RunningFire.3"));
            this.Textures.Add("Player.StandingFire.1", LevelContentManager.Load<Texture2D>("Player\\Player.StandingFire.1"));

            this.Textures.Add("Weapons.Buster", LevelContentManager.Load<Texture2D>("Weapons\\Weapons.Buster"));

            this.Textures.Add("Enemy.Can.1", LevelContentManager.Load<Texture2D>("Enemy\\Enemy.Can.1"));
            this.Textures.Add("Enemy.Can.2", LevelContentManager.Load<Texture2D>("Enemy\\Enemy.Can.2"));
            this.Textures.Add("Enemy.Can.3", LevelContentManager.Load<Texture2D>("Enemy\\Enemy.Can.3"));
            this.Textures.Add("Enemy.Can.4", LevelContentManager.Load<Texture2D>("Enemy\\Enemy.Can.4"));
            this.Textures.Add("Enemy.Can.5", LevelContentManager.Load<Texture2D>("Enemy\\Enemy.Can.5"));
            this.Textures.Add("Enemy.Can.6", LevelContentManager.Load<Texture2D>("Enemy\\Enemy.Can.6"));

            this.Textures.Add("Explosions.Explode.1", LevelContentManager.Load<Texture2D>("Explosions\\Explosions.Explode.1"));
            this.Textures.Add("Explosions.Explode.2", LevelContentManager.Load<Texture2D>("Explosions\\Explosions.Explode.2"));
            this.Textures.Add("Explosions.Explode.3", LevelContentManager.Load<Texture2D>("Explosions\\Explosions.Explode.3"));
            this.Textures.Add("Explosions.Explode.4", LevelContentManager.Load<Texture2D>("Explosions\\Explosions.Explode.4"));

            this.Textures.Add("Powerup.ExtraLife", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.ExtraLife"));
            this.Textures.Add("Powerup.EnergyTank.1", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.EnergyTank.1"));
            this.Textures.Add("Powerup.EnergyTank.2", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.EnergyTank.2"));
            this.Textures.Add("Powerup.SmallHealth.1", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.SmallHealth.1"));
            this.Textures.Add("Powerup.SmallHealth.2", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.SmallHealth.2"));
            this.Textures.Add("Powerup.LargeHealth.1", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.LargeHealth.1"));
            this.Textures.Add("Powerup.LargeHealth.2", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.LargeHealth.2"));
            this.Textures.Add("Powerup.SmallWeaponRefill.1", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.SmallWeaponRefill.1"));
            this.Textures.Add("Powerup.SmallWeaponRefill.2", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.SmallWeaponRefill.2"));
            this.Textures.Add("Powerup.LargeWeaponRefill.1", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.LargeWeaponRefill.1"));
            this.Textures.Add("Powerup.LargeWeaponRefill.2", LevelContentManager.Load<Texture2D>("Powerup\\Powerup.LargeWeaponRefill.2"));

            this.SoundEffects.Add("Player.SFX.Land", LevelContentManager.Load<SoundEffect>("Player\\SFX\\Player.SFX.Land"));
            this.SoundEffects.Add("Player.SFX.TookDamage", LevelContentManager.Load<SoundEffect>("Player\\SFX\\Player.SFX.TookDamage"));
            this.SoundEffects.Add("Player.SFX.Death", LevelContentManager.Load<SoundEffect>("Player\\SFX\\Player.SFX.Death"));

            this.SoundEffects.Add("Player.Meter.SFX.Fill", LevelContentManager.Load<SoundEffect>("Player\\Meter\\SFX\\Player.Meter.SFX.Fill"));

            this.SoundEffects.Add("Enemy.SFX.TookDamage", LevelContentManager.Load<SoundEffect>("Enemy\\SFX\\Enemy.SFX.TookDamage"));

            this.SoundEffects.Add("Weapons.SFX.BusterShoot", LevelContentManager.Load<SoundEffect>("Weapons\\SFX\\Weapons.SFX.BusterShoot"));

            this.SoundEffects.Add("Powerup.SFX.EnergyTank", LevelContentManager.Load<SoundEffect>("Powerup\\SFX\\Powerup.SFX.EnergyTank"));
        }

        public virtual void UnloadContent(ContentManager content)
        {
            LevelContentManager.Unload();
            LevelContentManager.Dispose();

            foreach (Texture2D tx in this.Textures.Values)
            {
                tx.Dispose();
            }
            foreach (SoundEffect sfx in this.SoundEffects.Values)
            {
                sfx.Dispose();
            }
            this.Textures.Clear();
            this.SoundEffects.Clear();
        }

        internal PlayerObject PlayerObject { get; set; }

        internal DateTime LevelStartDateTime { get; private set; }
        internal GameTime CurrentTickGameTime { get; private set; }
        internal long CurrentTickNumber { get; private set; }
        internal long CurrentRawTickNumber { get; private set; }

        private Random m_cachedRandom = null;
        private long m_cachedRandomTick = -1;

        internal Random Random
        {
            get
            {
                if (m_cachedRandom == null ||
                    m_cachedRandomTick != CurrentTickNumber)
                {
                    m_cachedRandom = new Random(Convert.ToInt32((LevelStartDateTime.Ticks + CurrentTickNumber) & 0x7fffffff));
                    m_cachedRandomTick = CurrentTickNumber;
                }
                return m_cachedRandom;
            }
        }

        internal delegate void AlternateTickProcessHandler();

        internal AlternateTickProcessHandler AlternateTickProcess { get; set; }

        internal void Tick(GameTime gameTime)
        {
            this.CurrentRawTickNumber++;
            if (this.AlternateTickProcess != null)
            {
                this.AlternateTickProcess();
            }
            else
            {
                this.CurrentTickGameTime = gameTime;
                this.CurrentTickNumber++;

                LevelGameObjectCollection gox = (LevelGameObjectCollection)this.GameObjects;
                gox.InTick = true;
                for (int prio = 1; prio <= (int)TickPriority.MAXIMUM; prio++)
                {
                    foreach (GameObject obj in this.GameObjectsByTickPriority.GetObjectsAtPriority(prio))
                    {
                        obj.Tick();
                    }
                }
                gox.InTick = false;
                while (gox.ObjectsToRemoveAfterTick.Count > 0)
                    gox.Remove(gox.ObjectsToRemoveAfterTick.Dequeue());
                while (gox.ObjectsToInsertAfterTick.Count > 0)
                    gox.Add(gox.ObjectsToInsertAfterTick.Dequeue());
            }

            this.CameraLocation = this.PlayerObject.BoundingBox.Center;
            this.CurrentCameraTopLeft = UpdateCamera();
            this.CurrentCameraArea = new Rectangle(this.CurrentCameraTopLeft.X, this.CurrentCameraTopLeft.Y,
                this.ScreenSize.X, this.ScreenSize.Y);
        }

        protected virtual Point UpdateCamera()
        {
            Rectangle screenViewRect = new Rectangle(this.CameraLocation.X - (this.ScreenSize.X / 2), this.CameraLocation.Y - (this.ScreenSize.Y / 2),
                this.ScreenSize.X, this.ScreenSize.Y);
            if (screenViewRect.Bottom > this.CameraBounds.Bottom)
            {
                screenViewRect = new Rectangle(screenViewRect.X, screenViewRect.Y - (screenViewRect.Bottom - this.CameraBounds.Bottom),
                    this.ScreenSize.X, this.ScreenSize.Y);
            }
            if (screenViewRect.Right > this.CameraBounds.Right)
            {
                screenViewRect = new Rectangle(screenViewRect.X - (screenViewRect.Right - this.CameraBounds.Right), screenViewRect.Y,
                    this.ScreenSize.X, this.ScreenSize.Y);
            }
            if (screenViewRect.Top < this.CameraBounds.Top)
            {
                screenViewRect = new Rectangle(screenViewRect.X, this.CameraBounds.Top, this.ScreenSize.X, this.ScreenSize.Y);
            }
            if (screenViewRect.Left < this.CameraBounds.Left)
            {
                screenViewRect = new Rectangle(this.CameraBounds.Left, screenViewRect.Y, this.ScreenSize.X, this.ScreenSize.Y);
            }
            return screenViewRect.Location;
        }

        internal void Draw(SpriteBatch spriteBatch)
        {
            Rectangle curCameraRect = this.CurrentCameraArea;
            foreach (GameObject obj in this.GameObjects.GetObjectsIntersecting(curCameraRect))
            {
                obj.Draw(this.CurrentCameraTopLeft, spriteBatch);
            }

            if (this.PlayerObject != null)
            {
                this.PlayerObject.DrawHealthMeter(spriteBatch);
            }
        }
    }
}
