﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;

namespace WindowsGame1.GameObjects.Enemies
{
    class CanEnemy : Enemy
    {
        public CanEnemy()
        {
            this.TextureName = CanEnemy.Textures[0];
            this.CurrentTexOfs = 0;
            this.BoundingBox = new Rectangle(0, 0, 16, 16);
            this.CurrentHp = 1;
            this.PlayerCollisionDamage = 4.0f;
        }


        private static readonly string[] Textures = {
                                              "Enemy.Can.1",
                                              "Enemy.Can.2",
                                              "Enemy.Can.3",
                                              "Enemy.Can.4",
                                              "Enemy.Can.5",
                                              "Enemy.Can.6",
                                          };
        private int CurrentTexOfs = 0;

        private enum MoveDirection
        {
            Up = 0,
            Down = 1,
            Left = 2,
            Right = 3,
            MAXIMUM = 3
        }

        private const float MovementSpeed = 0.3f;
        private const int ChangeDirectionInterval = 8;
        private MoveDirection CurrentDirection = MoveDirection.Down;
        private int TicksUntilNextChangeDir = 0;

        private RectangleF GetNewBoundingBox(RectangleF initial, MoveDirection dir, float distance)
        {
            switch (dir)
            {
                default:
                case MoveDirection.Up:
                    return new RectangleF(initial.X, initial.Y - distance, initial.Width, initial.Height);
                case MoveDirection.Down:
                    return new RectangleF(initial.X, initial.Y + distance, initial.Width, initial.Height);
                case MoveDirection.Left:
                    return new RectangleF(initial.X - distance, initial.Y, initial.Width, initial.Height);
                case MoveDirection.Right:
                    return new RectangleF(initial.X + distance, initial.Y, initial.Width, initial.Height);
            }
        }

        internal override void Tick()
        {
            if ((this.GameLevel.CurrentTickNumber % 5) == 0)
            {
                CurrentTexOfs++;
                if (CurrentTexOfs >= CanEnemy.Textures.Length)
                    CurrentTexOfs = 0;
                this.TextureName = CanEnemy.Textures[CurrentTexOfs];
            }

            TicksUntilNextChangeDir--;
            if (TicksUntilNextChangeDir <= 0)
            {
                Vector2 playerPointVec = this.GameLevel.PlayerObject.BoundingBoxF.Center;

                float minResultingDist = 9999999.0f;
                MoveDirection minResultingDir = MoveDirection.Down;

                for (int i = 0; i <= (int)MoveDirection.MAXIMUM; i++)
                {
                    RectangleF resultingBox = GetNewBoundingBox(this.BoundingBoxF, (MoveDirection)i, MovementSpeed * ChangeDirectionInterval);
                    Vector2 resultingVec = resultingBox.Center;
                    float resultingDist = Vector2.DistanceSquared(playerPointVec, resultingVec);
                    if (resultingDist < minResultingDist)
                    {
                        minResultingDist = resultingDist;
                        minResultingDir = (MoveDirection)i;
                    }
                }
                CurrentDirection = minResultingDir;
                TicksUntilNextChangeDir = ChangeDirectionInterval;
            }
            this.BoundingBoxF = GetNewBoundingBox(this.BoundingBoxF, CurrentDirection, MovementSpeed);
            
            //if ((this.GameLevel.CurrentTickNumber % 2) == 0)
            //{
            //    Point playerPoint = this.GameLevel.PlayerObject.BoundingBox.Center;
            //    Point myPoint = this.BoundingBox.Center;

            //    int moveX = 0;
            //    int moveY = 0;

            //    if (playerPoint.Y < myPoint.Y)
            //    {
            //        moveY = -1;
            //    }
            //    else if (playerPoint.Y > myPoint.Y)
            //    {
            //        moveY = 1;
            //    }

            //    if (playerPoint.X < myPoint.X)
            //    {
            //        moveX = -1;
            //    }
            //    else if (playerPoint.X > myPoint.X)
            //    {
            //        moveX = 1;
            //    }

            //    this.BoundingBox = new Rectangle(this.BoundingBox.X + moveX, this.BoundingBox.Y + moveY, this.BoundingBox.Width, this.BoundingBox.Height);
            //}
            base.Tick();
        }
    }
}
