﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace WindowsGame1.GameObjects.Enemies
{
    class CanEnemySpawner : GameObject
    {
        // Can enemies have special spawn points .. they can spawn even while onscreen, but only at a certain maximum rate,
        // and only if there are no more than 3 active Can enemies around.

        public CanEnemySpawner()
        {
            this.TickPriority = TickPriority.Enemy;
            this.EnemyToSpawn = new CanEnemy();
            this.BoundingBox = new Rectangle(0, 0, EnemyToSpawn.BoundingBox.Width, EnemyToSpawn.BoundingBox.Height);
        }

        public Point Location
        {
            get { return new Point(this.BoundingBox.Center.X, this.BoundingBox.Bottom); }
            set
            {
                this.BoundingBox = new Rectangle(value.X - this.BoundingBox.Width / 2, value.Y - this.BoundingBox.Height,
                    this.BoundingBox.Width, this.BoundingBox.Height);
            }
        }

        private class CanEnemyIndex : CollectionIndex<GameObject>
        {
            private List<CanEnemy> m_canEnemies = new List<CanEnemy>();
            private List<CanEnemy> m_pendingEnemies = new List<CanEnemy>();

            internal override void AddItem(GameObject obj)
            {
                if (obj is CanEnemy)
                {
                    m_canEnemies.Add((CanEnemy)obj);
                    m_pendingEnemies.Remove((CanEnemy)obj);
                }
            }

            internal override void RemoveItem(GameObject obj)
            {
                if (obj is CanEnemy)
                    m_canEnemies.Remove((CanEnemy)obj);
            }

            internal override void ClearItems(IEnumerable<GameObject> objs)
            {
                m_canEnemies.Clear();
            }

            internal void AddPendingEnemy(CanEnemy obj)
            {
                m_pendingEnemies.Add(obj);
            }

            internal int Count
            {
                get { return m_canEnemies.Count + m_pendingEnemies.Count; }
            }
        }

        protected override void OnGameLevelChanged(EventArgs e)
        {
            if (this.GameLevel != null)
            {
                CanEnemyIndex idx = this.GameLevel.GameObjects.GetIndex<CanEnemyIndex>();
                if (idx == null)
                {
                    idx = new CanEnemyIndex();
                    this.GameLevel.GameObjects.AttachIndex(idx);
                }
                CurrentLevelIndex = idx;
            }
            else
            {
                CurrentLevelIndex = null;
            }

            base.OnGameLevelChanged(e);
        }

        private long LastSpawnTick = -10000;
        private const long MinSpawnIntervalTicks = 60 * 3;
        private const int MaxEnemiesUpAtOnce = 3;
        private CanEnemy EnemyToSpawn;
        private CanEnemyIndex CurrentLevelIndex;


        private int GetCountOfActiveCanEnemies()
        {
            // TODO: implement me
            return CurrentLevelIndex.Count;
        }

        internal override void Tick()
        {
            bool isOnscreen = this.GameLevel.CurrentCameraArea.Intersects(this.BoundingBox);
            if (isOnscreen &&
                GetCountOfActiveCanEnemies() < MaxEnemiesUpAtOnce &&
                this.GameLevel.CurrentTickNumber >= LastSpawnTick + MinSpawnIntervalTicks)
            {
                CanEnemy spawningEnemy = EnemyToSpawn;
                EnemyToSpawn = new CanEnemy();

                spawningEnemy.PositionOnSpawnerBoundingBox(this.BoundingBox);
                this.GameLevel.GameObjects.Add(spawningEnemy);
                CurrentLevelIndex.AddPendingEnemy(spawningEnemy);
                LastSpawnTick = this.GameLevel.CurrentTickNumber;
            }
            base.Tick();
        }

        internal override void Draw(Point point, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch)
        {
            // Nothing to draw.
        }
    }
}
