﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
using WindowsGame1.GameObjects.Powerups;

namespace WindowsGame1.GameObjects.Enemies
{
    class Enemy : GameObject
    {
        public Enemy()
        {
            this.PlayerCollisionDamage = 2.0f;
            this.CanSpawnPowerupOnDeath = true;
            this.DespawnWhenOffscreen = true;
            this.TickPriority = TickPriority.Enemy;
        }

        public Point Location
        {
            get { return new Point(this.BoundingBox.Center.X, this.BoundingBox.Bottom); }
            set
            {
                this.BoundingBox = new Rectangle(value.X - this.BoundingBox.Width / 2, value.Y - this.BoundingBox.Height,
                    this.BoundingBox.Width, this.BoundingBox.Height);
            }
        }

        public float PlayerCollisionDamage { get; set; }

        private int m_currentHp;

        public int CurrentHp
        {
            get { return m_currentHp; }
            set
            {
                if (value != m_currentHp)
                {
                    m_currentHp = value;
                    if (m_currentHp == 0)
                        Die();
                }
            }
        }

        internal override void Tick()
        {
            Rectangle ccaExpanded = this.GameLevel.CurrentCameraArea;
            ccaExpanded = new Rectangle(ccaExpanded.Left - 32, ccaExpanded.Top - 32, ccaExpanded.Width + 64, ccaExpanded.Height + 64);
            if (this.DespawnWhenOffscreen &&
                !ccaExpanded.Intersects(this.BoundingBox))
            {
                this.GameLevel.GameObjects.Remove(this);
            }
        }

        protected bool DespawnWhenOffscreen { get; set; }
        public bool CanSpawnPowerupOnDeath { get; protected set; }

        protected virtual void Die()
        {
            SmallExplosion expObj = new SmallExplosion(this.BoundingBox.Center);
            this.GameLevel.GameObjects.Add(expObj);
            if (CanSpawnPowerupOnDeath)
            {
                MaybeSpawnPowerup();
            }
            this.GameLevel.GameObjects.Remove(this);
        }

        private class PowerupDropChanceDefinition
        {
            public Type PowerupType { get; set; }
            public int DropChance { get; set; }
        }

        private static readonly PowerupDropChanceDefinition[] PowerupDropChances = new[] {
            new PowerupDropChanceDefinition() { PowerupType = typeof(ExtraLife), DropChance = 1 },
            new PowerupDropChanceDefinition() { PowerupType = typeof(LargeHealthPellet), DropChance = 4 },
            new PowerupDropChanceDefinition() { PowerupType = typeof(SmallHealthPellet), DropChance = 15 },
            new PowerupDropChanceDefinition() { PowerupType = typeof(LargeWeaponPellet), DropChance = 5 },
            new PowerupDropChanceDefinition() { PowerupType = typeof(SmallWeaponPellet), DropChance = 25 },
            //new PowerupDropChanceDefinition() { PowerupType = typeof(EnergyTank), DropChance = 127 },
        };

        private void MaybeSpawnPowerup()
        {
            int powerupNum = this.GameLevel.Random.Next(0, 127);
            foreach (PowerupDropChanceDefinition pdcd in PowerupDropChances)
            {
                if (powerupNum <= pdcd.DropChance)
                {
                    Powerup p = (Powerup)Activator.CreateInstance(pdcd.PowerupType);
                    p.Expires = true;
                    p.Location = this.BoundingBox.Center;
                    this.GameLevel.GameObjects.Add(p);
                    return;
                }
                else
                {
                    powerupNum -= pdcd.DropChance;
                }
            }
        }

        internal override void ProjectileIntersect(Projectile proj)
        {
            this.GameLevel.SoundEffects["Enemy.SFX.TookDamage"].Play();
            this.CurrentHp -= proj.DamageDealt;
            proj.Consume();
            //proj.Reflect();
        }

        public virtual void CollideWithPlayer(PlayerObject player)
        {
            player.DealDamage(this.PlayerCollisionDamage);
        }

        internal void PositionOnSpawnerBoundingBox(Rectangle bbox)
        {
            this.BoundingBox = bbox;
        }
    }
}
