﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace WindowsGame1.GameObjects.Enemies
{
    class EnemySpawner<T> : GameObject
        where T : Enemy, new()
    {
        public EnemySpawner()
        {
            this.TickPriority = TickPriority.Enemy;
            this.EnemyToSpawn = new T();
            this.BoundingBox = new Rectangle(0, 0, EnemyToSpawn.BoundingBox.Width, EnemyToSpawn.BoundingBox.Height);
        }

        public Point Location
        {
            get { return new Point(this.BoundingBox.Center.X, this.BoundingBox.Bottom); }
            set
            {
                this.BoundingBox = new Rectangle(value.X - this.BoundingBox.Width / 2, value.Y - this.BoundingBox.Height,
                    this.BoundingBox.Width, this.BoundingBox.Height);
            }
        }

        private bool WasOnscreenLastFrame = false;
        private T EnemyToSpawn;
        private T LastEnemySpawned;

        internal override void Tick()
        {
            // Don't spawn if offscreen, or if the last enemy spawned is still around
            bool isOnscreen = this.GameLevel.CurrentCameraArea.Intersects(this.BoundingBox);
            if (isOnscreen &&
                (LastEnemySpawned == null || LastEnemySpawned.GameLevel != this.GameLevel) &&
                !WasOnscreenLastFrame)
            {
                T spawningEnemy = EnemyToSpawn;
                EnemyToSpawn = new T();

                LastEnemySpawned = spawningEnemy;
                spawningEnemy.PositionOnSpawnerBoundingBox(this.BoundingBox);
                this.GameLevel.GameObjects.Add(spawningEnemy);
            }
            WasOnscreenLastFrame = isOnscreen;
            base.Tick();
        }

        internal override void Draw(Point point, Microsoft.Xna.Framework.Graphics.SpriteBatch spriteBatch)
        {
            // Nothing to draw.
        }
    }
}
