﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;

namespace WindowsGame1.GameObjects
{
    class GameObject
    {
        private GameLevel m_gameLevel;

        public GameLevel GameLevel
        {
            get { return m_gameLevel; }
            set
            {
                if (value != m_gameLevel)
                {
                    m_gameLevel = value;
                    OnGameLevelChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnGameLevelChanged(EventArgs e)
        {
        }

        private Rectangle m_boundingBox;
        private RectangleF m_boundingBoxF;

        public Rectangle BoundingBox
        {
            get { return m_boundingBox; }
            protected set
            {
                m_boundingBox = value;
                m_boundingBoxF = new RectangleF(value.X, value.Y, value.Width, value.Height);
            }
        }

        public RectangleF BoundingBoxF
        {
            get { return m_boundingBoxF; }
            protected set
            {
                m_boundingBoxF = value;
                m_boundingBox = value.ToRectangle();
            }
        }

        public string TextureName { get; set; }

        private TickPriority m_tickPrio = TickPriority.None;

        public TickPriority TickPriority
        {
            get { return m_tickPrio; }
            protected set
            {
                if (value != m_tickPrio)
                {
                    OnTickPriorityChanging(EventArgs.Empty);
                    m_tickPrio = value;
                    OnTickPriorityChanged(EventArgs.Empty);
                }
            }
        }

        protected virtual void OnTickPriorityChanging(EventArgs e)
        {
            EventHandler temp = TickPriorityChanging;
            if (temp != null)
            {
                temp(this, e);
            }
        }

        public event EventHandler TickPriorityChanging;

        protected virtual void OnTickPriorityChanged(EventArgs e)
        {
            EventHandler temp = TickPriorityChanged;
            if (temp != null)
            {
                temp(this, e);
            }
        }

        public event EventHandler TickPriorityChanged;

        protected float VerticalVelocity = 0.0f;
        protected float MaximumVerticalVelocity = 6.0f;

        protected bool ApplyGravity()
        {
            bool touchedSpike;
            return ApplyGravity(out touchedSpike);
        }

        protected virtual bool ApplyGravity(out bool touchedSpike)
        {
            bool onSolidGround = false;
            touchedSpike = false;
            if (VerticalVelocity >= 0.0f)
            {
                bool onNonSpikeTile = false;
                bool canTouchSpike = false;
                foreach (GameObject obj in this.GameLevel.GameObjects.GetObjectsIntersecting(
                    new Rectangle(this.BoundingBox.Left, this.BoundingBox.Bottom, this.BoundingBox.Width, 1)))
                {
                    if (obj is FloorTile)
                    {
                        onSolidGround = true;
                        if (obj is SpikeTile)
                        {
                            canTouchSpike = true;
                            int objcx = this.BoundingBox.Center.X;
                            if (objcx >= obj.BoundingBox.Left && objcx <= obj.BoundingBox.Right)
                            {
                                touchedSpike = true;
                            }
                        }
                        else
                        {
                            onNonSpikeTile = true;
                        }
                        //break;
                    }
                }
                if (!onNonSpikeTile && canTouchSpike)
                    touchedSpike = true;
            }
            if (!onSolidGround)
            {
                VerticalVelocity += this.GameLevel.Gravity.Y;
            }
            if (VerticalVelocity > MaximumVerticalVelocity)
            {
                VerticalVelocity = MaximumVerticalVelocity;
            }
            else if (VerticalVelocity < 0 - MaximumVerticalVelocity)
            {
                VerticalVelocity = 0 - MaximumVerticalVelocity;
            }
            if (VerticalVelocity != 0.0f)
            {
                float velOverlapAdjust = (VerticalVelocity > 0.0f) ? -1.0f : 1.0f;
                bool floorIntersect;
                Rectangle proposedBoundingBox;
                do
                {
                    int intVertVel = Convert.ToInt32(Math.Round(VerticalVelocity));
                    proposedBoundingBox = new Rectangle(this.BoundingBox.Left, this.BoundingBox.Top + intVertVel, this.BoundingBox.Width, this.BoundingBox.Height);
                    floorIntersect = false;
                    bool onNonSpikeTile = false;
                    bool canTouchSpike = false;
                    foreach (GameObject obj in this.GameLevel.GameObjects.GetObjectsIntersecting(proposedBoundingBox))
                    {
                        if (obj is FloorTile)
                        {
                            floorIntersect = true;
                            if (obj is SpikeTile)
                            {
                                canTouchSpike = true;
                                int objcx = this.BoundingBox.Center.X;
                                if (objcx >= obj.BoundingBox.Left && objcx <= obj.BoundingBox.Right)
                                {
                                    touchedSpike = true;
                                }
                            }
                            else
                            {
                                onNonSpikeTile = true;
                            }
                        }
                    }
                    if (!onNonSpikeTile && canTouchSpike)
                        touchedSpike = true;
                    if (floorIntersect)
                    {
                        if (Math.Abs(VerticalVelocity) < Math.Abs(velOverlapAdjust) && VerticalVelocity != 0.0f)
                            VerticalVelocity = 0.0f;
                        else
                            VerticalVelocity += velOverlapAdjust;
                    }
                } while (floorIntersect);
                this.BoundingBox = proposedBoundingBox;
            }
            return !onSolidGround; // || Math.Abs(VerticalVelocity) >= 0.0f;
        }

        internal virtual void Tick()
        {
        }

        internal virtual void Draw(Point point, SpriteBatch spriteBatch)
        {
            spriteBatch.Draw(this.GameLevel.Textures[this.TextureName], new Rectangle(this.BoundingBox.X - point.X,
                this.BoundingBox.Y - point.Y, this.BoundingBox.Width, this.BoundingBox.Height), Color.White);
        }

        internal virtual void ProjectileIntersect(Projectile proj)
        {
        }
    }
}
