﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;
using WindowsGame1.GameObjects.Powerups;
using WindowsGame1.GameObjects.Enemies;

namespace WindowsGame1.GameObjects
{
    class PlayerObject : GameObject
    {
        public PlayerObject()
        {
            this.TickPriority = TickPriority.Player;
            this.BoundingBox = new Rectangle(0, 0, 16, 22);
        }

        public Point Location
        {
            get { return new Point(this.BoundingBox.Center.X, this.BoundingBox.Bottom); }
            set
            {
                this.BoundingBox = new Rectangle(value.X - this.BoundingBox.Width / 2, value.Y - this.BoundingBox.Height, this.BoundingBox.Width, this.BoundingBox.Height);
            }
        }

        //public new Rectangle BoundingBox
        //{
        //    get { return base.BoundingBox; }
        //    set { base.BoundingBox = value; }
        //}

        private float HorizontalVelocity = 0.0f;
        private float MaximumHorizontalVelocity = 6.0f;
        private int OnGroundFramesAgo = 0;
        private bool LastFrameJumpPressed = false;
        private bool LastFrameJumpHeld = false;
        private bool LastFrameFirePressed = false;
        private bool FacingRight = true;
        private int ProjectilesInFlight = 0;
        private long LastFireTick = -99999;
        private float StunnedBackstepSpeed = 1.0f;

        private bool WasFallingLastFrame = false;

        internal override void Tick()
        {
            if (IsDead)
                return;

            KeyboardState ks = Keyboard.GetState();

            bool touchedSpike;
            bool standingOnGround = !ApplyGravity(out touchedSpike);

            float standFriction = 0.0f;
            if (standingOnGround)
            {
                foreach (GameObject obj in this.GameLevel.GameObjects.GetObjectsIntersecting(new Rectangle(this.BoundingBox.X, this.BoundingBox.Bottom, this.BoundingBox.Width, 1)))
                {
                    if (obj is FloorTile)
                    {
                        standFriction = Math.Max(standFriction, ((FloorTile)obj).Friction);
                    }
                }
            }
            if (!standingOnGround)
            {
                OnGroundFramesAgo += 1;
            }
            else
            {
                OnGroundFramesAgo = 0;
                if (this.HorizontalVelocity != 0.0f)
                {
                    float xVel = this.HorizontalVelocity;
                    if (xVel > 0 && !ks.IsKeyDown(Keys.Right))
                        xVel = Math.Max(0.0f, xVel - standFriction);
                    else if (xVel < 0 && !ks.IsKeyDown(Keys.Left))
                        xVel = Math.Min(0.0f, xVel + standFriction);
                    this.HorizontalVelocity = xVel;
                }
            }
            if (this.CurrentlyStunned)
            {
                if (FacingRight)
                {
                    this.HorizontalVelocity = 0 - StunnedBackstepSpeed;
                }
                else
                {
                    this.HorizontalVelocity = StunnedBackstepSpeed;
                }
            }

            this.HorizontalVelocity = Math.Max(0 - MaximumHorizontalVelocity, Math.Min(MaximumHorizontalVelocity, this.HorizontalVelocity));

            int veloX = Convert.ToInt32(Math.Round(this.HorizontalVelocity));

            bool isIntersect;
            do
            {
                Rectangle proposedBoundingBox = new Rectangle(this.BoundingBox.X + veloX,
                    this.BoundingBox.Y, this.BoundingBox.Width, this.BoundingBox.Height);

                isIntersect = false;
                bool onNonSpikeTile = false;
                bool canTouchSpike = false;
                foreach (GameObject obj in this.GameLevel.GameObjects.GetObjectsIntersecting(proposedBoundingBox))
                {
                    if (obj is FloorTile)
                    {
                        isIntersect = true;
                        if (obj is SpikeTile)
                        {
                            canTouchSpike = true;
                            int objcy = this.BoundingBox.Center.Y;
                            if (objcy >= obj.BoundingBox.Top && objcy <= obj.BoundingBox.Bottom)
                            {
                                touchedSpike = true;
                            }
                        }
                        else
                        {
                            onNonSpikeTile = true;
                        }
                        //break;
                    }
                }
                if (!onNonSpikeTile && canTouchSpike)
                    touchedSpike = true;

                if (isIntersect)
                {
                    if (veloX != 0)
                    {
                        if (veloX > 0)
                        {
                            veloX -= 1;
                        }
                        else if (veloX < 0)
                        {
                            veloX += 1;
                        }
                        this.HorizontalVelocity = veloX;
                    }
                }
                if (!isIntersect)
                    this.BoundingBox = proposedBoundingBox;
            } while (isIntersect);

            if (!standingOnGround)
            {
                WasFallingLastFrame = true;
            }
            else
            {
                if (WasFallingLastFrame)
                    this.GameLevel.SoundEffects["Player.SFX.Land"].Play();
                WasFallingLastFrame = false;
            }

            if (touchedSpike)
            {
                this.DealDamage(999.0f);
            }
            else if (this.BoundingBox.Top > this.GameLevel.CameraBounds.Bottom + 50)
            {
                // Fell off bottom of screen
                NoDeathExplosion = true;
                this.DealDamage(999.0f, true);
            }
            else
            {
                foreach (GameObject obj in this.GameLevel.GameObjects.GetObjectsIntersecting(this.BoundingBox))
                {
                    if (obj is Powerup)
                    {
                        ((Powerup)obj).CollideWithPlayer(this);
                    }
                    else if (obj is Enemy)
                    {
                        ((Enemy)obj).CollideWithPlayer(this);
                    }
                }
            }

            //this.TextureName = isIntersect ? "playerobject2" : "playerobject";

            bool isRunning = false;

            if (!CurrentlyStunned)
            {
                if (ks.IsKeyDown(Keys.Right))
                {
                    FacingRight = true;
                    isRunning = true;
                    float xVel = this.HorizontalVelocity;
                    if (standingOnGround)
                        xVel += 2.0f;
                    else
                        xVel += 0.2f;
                    xVel = Math.Min(2.0f, xVel);
                    this.HorizontalVelocity = xVel;
                }
                else if (ks.IsKeyDown(Keys.Left))
                {
                    FacingRight = false;
                    isRunning = true;
                    float xVel = this.HorizontalVelocity;
                    if (standingOnGround)
                        xVel -= 2.0f;
                    else
                        xVel -= 0.2f;
                    xVel = Math.Max(-2.0f, xVel);
                    this.HorizontalVelocity = xVel;
                }

                bool firePressed = ks.IsKeyDown(Keys.LeftControl);
                if (firePressed && !LastFrameFirePressed && ProjectilesInFlight < 3)
                {
                    BusterProjectile proj;
                    if (FacingRight)
                    {
                        proj = new BusterProjectile(this.BoundingBox.Right + 8, this.BoundingBox.Top + 9, BusterProjectile.FireSpeed);
                    }
                    else
                    {
                        proj = new BusterProjectile(this.BoundingBox.Left - 8, this.BoundingBox.Top + 9, 0 - BusterProjectile.FireSpeed);
                    }
                    proj.ProjectileDead += new EventHandler(proj_ProjectileDead);
                    this.GameLevel.GameObjects.Add(proj);
                    ProjectilesInFlight++;
                    this.GameLevel.SoundEffects["Weapons.SFX.BusterShoot"].Play();
                    LastFireTick = this.GameLevel.CurrentTickNumber;
                }
                LastFrameFirePressed = firePressed;

                bool jumpPressed = ks.IsKeyDown(Keys.Space);

                if (standingOnGround && jumpPressed && !LastFrameJumpPressed)
                {
                    this.VerticalVelocity -= 4.0f;
                    LastFrameJumpHeld = true;
                }
                else if (!standingOnGround && jumpPressed && LastFrameJumpHeld && OnGroundFramesAgo < 9)
                {
                    if (Math.Abs(this.VerticalVelocity) < 1.0f)
                    {
                        LastFrameJumpHeld = false;
                    }
                    else
                    {
                        this.VerticalVelocity -= 4.0f;
                        LastFrameJumpHeld = true;
                    }
                }
                else
                {
                    LastFrameJumpHeld = false;
                }
                LastFrameJumpPressed = jumpPressed;
            }
            if (!CurrentlyStunned)
            {
                if (!standingOnGround)
                {
                    if (LastFireTick + 15 >= this.GameLevel.CurrentTickNumber)
                        this.BeginAnimation("InAirFire", !FacingRight);
                    else
                        this.BeginAnimation("InAir", !FacingRight);
                }
                else if (isRunning)
                {
                    if (LastFireTick + 15 >= this.GameLevel.CurrentTickNumber)
                        this.BeginAnimation("RunningFire", !FacingRight);
                    else
                        this.BeginAnimation("Running", !FacingRight);
                }
                else
                {
                    if (LastFireTick + 15 >= this.GameLevel.CurrentTickNumber)
                        this.BeginAnimation("StandingFire", !FacingRight);
                    else
                        this.BeginAnimation("Standing", !FacingRight);
                }
            }
            else
            {
                this.BeginAnimation("Stunned", !FacingRight);
            }
        }

        void proj_ProjectileDead(object sender, EventArgs e)
        {
            ProjectilesInFlight--;
        }

        private long m_animStartAt = 0;
        private string m_curAnimation = "Standing";
        private bool m_curFlipHoriz = false;
        private string m_lastActualFrame = "Player.Standing.1";

        private void BeginAnimation(string animationName, bool flipHoriz)
        {
            if (animationName != m_curAnimation || flipHoriz != m_curFlipHoriz)
            {
                m_animStartAt = this.GameLevel.CurrentTickNumber;
                m_curAnimation = animationName;
                m_curFlipHoriz = flipHoriz;
            }
        }

        internal override void Draw(Point point, SpriteBatch spriteBatch)
        {
            string actualFrame = "Player.Standing.1";
            if (!IsDead)
            {
                long framesSinceAnimStart = this.GameLevel.CurrentTickNumber - m_animStartAt;
                if (m_curAnimation == "InAir")
                {
                    actualFrame = "Player.InAir.1";
                }
                else if (m_curAnimation == "InAirFire")
                {
                    actualFrame = "Player.InAirFire.1";
                }
                else if (m_curAnimation == "Running")
                {
                    if (framesSinceAnimStart < 4)
                    {
                        actualFrame = "Player.Running.0";
                    }
                    else
                    {
                        long fmod = (framesSinceAnimStart % (7 * 4));
                        if (fmod > (7 * 3))
                            actualFrame = "Player.Running.1";
                        else if (fmod > (7 * 2))
                            actualFrame = "Player.Running.2";
                        else if (fmod > (7 * 1))
                            actualFrame = "Player.Running.3";
                        else
                            actualFrame = "Player.Running.2";
                    }
                }
                else if (m_curAnimation == "RunningFire")
                {
                    long fmod = (framesSinceAnimStart % (7 * 4));
                    if (fmod > (7 * 3))
                        actualFrame = "Player.RunningFire.1";
                    else if (fmod > (7 * 2))
                        actualFrame = "Player.RunningFire.2";
                    else if (fmod > (7 * 1))
                        actualFrame = "Player.RunningFire.3";
                    else
                        actualFrame = "Player.RunningFire.2";
                }
                else if (m_curAnimation == "Standing")
                {
                    long fmod = (framesSinceAnimStart % 240);
                    if (fmod > 230)
                        actualFrame = "Player.Standing.2";
                    else
                        actualFrame = "Player.Standing.1";
                }
                else if (m_curAnimation == "StandingFire")
                {
                    actualFrame = "Player.StandingFire.1";
                }
                else if (m_curAnimation == "Stunned")
                {
                    if ((this.GameLevel.CurrentTickNumber % 2) == 0)
                    {
                        actualFrame = "Player.Stunned.1";
                    }
                    else
                    {
                        actualFrame = "Player.TookDamage";
                    }
                }
                m_lastActualFrame = actualFrame;
            }
            else
            {
                actualFrame = m_lastActualFrame;
            }

            if (!IsDeadAndGone)
            {
                Rectangle drawRect = new Rectangle(this.BoundingBox.X - point.X,
                        this.BoundingBox.Y - point.Y, this.BoundingBox.Width, this.BoundingBox.Height);
                SpriteEffects eff = SpriteEffects.None;
                if (!FacingRight)
                {
                    eff = SpriteEffects.FlipHorizontally;
                }

                Texture2D tex = this.GameLevel.Textures[actualFrame];
                int texTop = this.BoundingBox.Bottom - Math.Min(24, tex.Height);
                int texLeft = this.BoundingBox.Center.X - (tex.Width / 2);

                if (!this.CurrentlyInvulnerable || this.CurrentlyStunned ||
                    (this.GameLevel.CurrentTickNumber % 2) == 0)
                {
                    spriteBatch.Draw(tex, new Rectangle(texLeft - point.X, texTop - point.Y, tex.Width, tex.Height), null, Color.White, 0.0f, new Vector2(0.0f, 0.0f), eff, 0);
                }
            }
        }

        public int Lives { get; set; }

        public const float MaxHp = 27.0f;

        private float m_currentHp = MaxHp;

        public float CurrentHp
        {
            get { return m_currentHp; }
            set
            {
                if (value != m_currentHp)
                {
                    m_currentHp = value;
                }
            }
        }

        internal void DrawHealthMeter(SpriteBatch spriteBatch)
        {
            Point meterPos = new Point(10, 10);

            Texture2D meterBg = this.GameLevel.Textures["Player.Meter.Background"];
            Texture2D meterPip = this.GameLevel.Textures["Player.Meter.HealthPip"];
            spriteBatch.Draw(meterBg, new Rectangle(meterPos.X, meterPos.Y, meterBg.Width, meterBg.Height), Color.Black);
            int pipsToDraw = Convert.ToInt32(Math.Ceiling(this.CurrentHp));
            for (int pn = 0; pn <= pipsToDraw; pn++)
            {
                spriteBatch.Draw(meterPip, new Rectangle(meterPos.X + 1, meterPos.Y + meterBg.Height - ((pn + 1) * 2), meterPip.Width, meterPip.Height), Color.White);
            }
        }

        public long InvulnerableUntilTick { get; private set; }
        private const long DamageInvulnerableTicks = 90;

        public bool CurrentlyInvulnerable
        {
            get { return (this.GameLevel.CurrentTickNumber <= this.InvulnerableUntilTick); }
        }

        public long StunnedUntilTick { get; private set; }
        public const long StunTicks = 20;

        public bool CurrentlyStunned
        {
            get { return (this.GameLevel.CurrentTickNumber <= this.StunnedUntilTick); }
        }

        private bool NoDeathExplosion = false;
        private bool IsDead = false;
        private bool IsDeadAndGone = false;

        public void DealDamage(float damageAmount)
        {
            DealDamage(damageAmount, false);
        }

        public void DealDamage(float damageAmount, bool bypassInvulnerability)
        {
            if (IsDead)
                return;

            if (!this.CurrentlyInvulnerable || bypassInvulnerability)
            {
                this.CurrentHp -= damageAmount;
                if (this.CurrentHp > 0.0f)
                {
                    this.InvulnerableUntilTick = this.GameLevel.CurrentTickNumber + DamageInvulnerableTicks;
                    this.StunnedUntilTick = this.GameLevel.CurrentTickNumber + StunTicks;

                    PlayerDamageSweat sw1 = new PlayerDamageSweat(new Point(this.BoundingBox.Center.X, this.BoundingBox.Top - 5));
                    PlayerDamageSweat sw2 = new PlayerDamageSweat(new Point(this.BoundingBox.Left - 2, this.BoundingBox.Top - 2));
                    PlayerDamageSweat sw3 = new PlayerDamageSweat(new Point(this.BoundingBox.Right + 2, this.BoundingBox.Top - 2));

                    this.GameLevel.GameObjects.Add(sw1);
                    this.GameLevel.GameObjects.Add(sw2);
                    this.GameLevel.GameObjects.Add(sw3);

                    this.GameLevel.SoundEffects["Player.SFX.TookDamage"].Play();
                }
                else
                {
                    this.IsDead = true;
                    MegaExplosion ex = new MegaExplosion();
                    if (NoDeathExplosion)
                        ex.Location = new Point(-999999, -99999);
                    else
                        ex.Location = this.BoundingBox.Center;
                    ex.MiniExplosionComplete += DeathMiniExplosionCompleted;
                    ex.ExplosionComplete += DeathExplosionCompleted;
                    this.GameLevel.GameObjects.Add(ex);
                }
            }
        }

        private void DeathMiniExplosionCompleted()
        {
            this.IsDeadAndGone = true;
        }

        private void DeathExplosionCompleted()
        {
        }

        public void HealDamage(float healAmount)
        {
            if (!IsDead)
            {
                if (this.CurrentHp == PlayerObject.MaxHp)
                    return;

                // How long is the heal sfx?
                SoundEffect healSfx = this.GameLevel.SoundEffects["Player.Meter.SFX.Fill"];
                int waitTicksPerHeal = Convert.ToInt32(Math.Ceiling(healSfx.Duration.TotalMilliseconds / (1000.0 / 60.0)));
                int waitTicksCurrentHeal = 0;

                GameLevel.AlternateTickProcessHandler h = null;
                h = delegate
                {
                    if (waitTicksCurrentHeal <= 0)
                    {
                        if (healAmount <= 0.0f || this.CurrentHp >= PlayerObject.MaxHp)
                        {
                            if (this.CurrentHp >= PlayerObject.MaxHp)
                            {
                                this.CurrentHp = PlayerObject.MaxHp;
                            }
                            this.GameLevel.AlternateTickProcess = null;
                        }
                        else
                        {
                            if (healAmount >= 1.0f)
                            {
                                this.CurrentHp += 1.0f;
                                healAmount -= 1.0f;
                            }
                            else
                            {
                                this.CurrentHp += healAmount;
                                healAmount = 0.0f;
                            }
                            healSfx.Play();
                            waitTicksCurrentHeal = waitTicksPerHeal;
                        }
                    }
                    waitTicksCurrentHeal--;
                };
                this.GameLevel.AlternateTickProcess = h;
            }
        }

        internal void RefillWeapon(float refillAmount)
        {
            if (!IsDead)
            {
                // TODO: implement me
            }
        }
    }


}
