﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Audio;

namespace WindowsGame1.GameObjects
{
    class Projectile : GameObject
    {
        public Projectile()
        {
            this.Reflected = false;
            this.TickPriority = TickPriority.Projectile;
        }

        public Vector2 Velocity { get; set; }
        public bool Reflected { get; set; }
        public int DamageDealt { get; protected set; }

        internal override void Tick()
        {
            this.BoundingBox = new Rectangle(this.BoundingBox.X + Convert.ToInt32(Math.Round(this.Velocity.X)),
                this.BoundingBox.Y + Convert.ToInt32(Math.Round(this.Velocity.Y)), this.BoundingBox.Width, this.BoundingBox.Height);
            if (!this.GameLevel.CurrentCameraArea.Intersects(this.BoundingBox))
            {
                this.GameLevel.GameObjects.Remove(this);
                OnProjectileDead(EventArgs.Empty);
                this.IsDead = true;
            }
            else
            {
                if (!Reflected)
                {
                    foreach (GameObject gobj in this.GameLevel.GameObjects.GetObjectsIntersecting(this.BoundingBox))
                    {
                        gobj.ProjectileIntersect(this);
                        if (this.IsDead || this.Reflected)
                            break;
                    }
                }
            }
        }

        private bool IsDead = false;

        protected virtual void OnProjectileDead(EventArgs e)
        {
            EventHandler temp = ProjectileDead;
            if (temp != null)
            {
                temp(this, e);
            }
        }

        public event EventHandler ProjectileDead;

        internal void Consume()
        {
            this.GameLevel.GameObjects.Remove(this);
            OnProjectileDead(EventArgs.Empty);
            this.IsDead = true;
        }

        internal void Reflect()
        {
            if (this.Velocity.X > 0)
            {
                this.Velocity = new Vector2(-6.0f, -6.0f);
                this.Reflected = true;
            }
            else
            {
                this.Velocity = new Vector2(6.0f, -6.0f);
                this.Reflected = true;
            }
        }
    }

}
