﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Collections.ObjectModel;

namespace WindowsGame1
{
    class IndexableCollection<T> : Collection<T>
    {
        private List<CollectionIndex<T>> Indexes = new List<CollectionIndex<T>>();

        internal void AttachIndex(CollectionIndex<T> idx)
        {
            Indexes.Add(idx);
            foreach (T obj in Items)
            {
                idx.AddItem(obj);
            }
        }

        internal TIdx GetIndex<TIdx>()
            where TIdx : CollectionIndex<T>
        {
            foreach (CollectionIndex<T> idx in Indexes)
            {
                if (idx is TIdx)
                    return (TIdx)idx;
            }
            return null;
        }

        protected override void InsertItem(int index, T item)
        {
            base.InsertItem(index, item);
            foreach (CollectionIndex<T> idx in Indexes)
                idx.AddItem(item);
        }

        protected override void RemoveItem(int index)
        {
            T item = Items[index];
            foreach (CollectionIndex<T> idx in Indexes)
                idx.RemoveItem(item);
            base.RemoveItem(index);
        }

        protected override void SetItem(int index, T item)
        {
            T oldItem = Items[index];
            foreach (CollectionIndex<T> idx in Indexes)
                idx.RemoveItem(oldItem);
            base.SetItem(index, item);
            foreach (CollectionIndex<T> idx in Indexes)
                idx.AddItem(item);
        }

        protected override void ClearItems()
        {
            foreach (CollectionIndex<T> idx in Indexes)
                idx.ClearItems(Items);
            base.ClearItems();
        }
    }
}
