﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsGame1.GameObjects;

namespace WindowsGame1
{
    internal class LevelGameObjectCollection : GameObjectCollection
    {
        internal LevelGameObjectCollection(GameLevel owner)
        {
            this.Owner = owner;
        }

        private GameLevel Owner;

        internal bool InTick = false;
        internal Queue<GameObject> ObjectsToRemoveAfterTick = new Queue<GameObject>();
        internal Queue<GameObject> ObjectsToInsertAfterTick = new Queue<GameObject>();

        protected override void InsertItem(int index, GameObject item)
        {
            if (InTick)
            {
                ObjectsToInsertAfterTick.Enqueue(item);
            }
            else
            {
                base.InsertItem(index, item);
                item.GameLevel = Owner;
            }
        }

        protected override void RemoveItem(int index)
        {
            GameObject obj = Items[index];
            if (InTick)
            {
                ObjectsToRemoveAfterTick.Enqueue(obj);
            }
            else
            {
                obj.GameLevel = null;
                base.RemoveItem(index);
            }
        }

        protected override void SetItem(int index, GameObject item)
        {
            GameObject oldObj = Items[index];
            if (item != oldObj)
            {
                oldObj.GameLevel = null;
            } 
            base.SetItem(index, item);
            if (item != oldObj)
            {
                item.GameLevel = Owner;
            }
        }

        protected override void ClearItems()
        {
            foreach (GameObject obj in this.Items)
            {
                obj.GameLevel = null;
            }
            base.ClearItems();
        }
    }

}
