﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using WindowsGame1.GameObjects;

namespace WindowsGame1
{
    internal class TickPriorityIndex : CollectionIndex<GameObject>
    {
        public TickPriorityIndex()
        {
            int maxTickPrio = (int)TickPriority.MAXIMUM;
            Index_ByTickPriority = new List<GameObject>[maxTickPrio + 1];
            for (int i = 0; i <= maxTickPrio; i++)
            {
                Index_ByTickPriority[i] = new List<GameObject>();
            }
        }

        private List<GameObject>[] Index_ByTickPriority;

        internal override void AddItem(GameObject obj)
        {
            obj.TickPriorityChanging += new EventHandler(obj_TickPriorityChanging);
            obj.TickPriorityChanged += new EventHandler(obj_TickPriorityChanged);

            obj_TickPriorityChanged(obj, EventArgs.Empty);
        }

        internal override void RemoveItem(GameObject obj)
        {
            obj.TickPriorityChanging -= new EventHandler(obj_TickPriorityChanging);
            obj.TickPriorityChanged -= new EventHandler(obj_TickPriorityChanged);

            obj_TickPriorityChanging(obj, EventArgs.Empty);
        }

        internal override void ClearItems(IEnumerable<GameObject> objs)
        {
            foreach (GameObject obj in objs)
            {
                obj.TickPriorityChanging -= new EventHandler(obj_TickPriorityChanging);
                obj.TickPriorityChanged -= new EventHandler(obj_TickPriorityChanged);
            }
            for (int i = 0; i < Index_ByTickPriority.Length; i++)
            {
                Index_ByTickPriority[i].Clear();
            }
        }

        void obj_TickPriorityChanged(object sender, EventArgs e)
        {
            GameObject obj = (GameObject)sender;
            Index_ByTickPriority[(int)obj.TickPriority].Add(obj);
        }

        void obj_TickPriorityChanging(object sender, EventArgs e)
        {
            GameObject obj = (GameObject)sender;
            Index_ByTickPriority[(int)obj.TickPriority].Remove(obj);
        }

        public IEnumerable<GameObject> GetObjectsAtPriority(int prio)
        {
            return Index_ByTickPriority[prio];
        }
    }
}
